clear all   % Clear memory
%
% Structural parameters
%
alpha	= 0.4;
delta	= 0.025;
rho     = 0.95;
beta	= 0.988;
hs		= 0.31;
sz		= 0.00217;
%
% Deterministic Steady state
%
ysk	= (1-beta*(1-delta))/(alpha*beta);
ksy	= 1/ysk;
si	= delta/ysk;
sc	= 1-si;
% Define:
%
% Y=[k(t+1) a(t+1) E_tc(t+1)]
%
% X=[y,c,i,h]
%
ny  = 3; % # of variables in vector Y
nx  = 4; % # of variables in vector X
ne  = 1; % # of fundamental shocks
nn  = 1; % # of expectation errors
%
% Initialize the Upsilon matrices
%
UX=zeros(nx,nx);
UY=zeros(nx,ny);
UE=zeros(nx,ne);
UN=zeros(nx,nn);

G0Y=zeros(ny,ny);
G1Y=zeros(ny,ny);
G0X=zeros(ny,nx);
G1X=zeros(ny,nx);
GE=zeros(ny,ne);
GN=zeros(ny,nn);
%
% Production function
%
UX(1,1)=1;
UX(1,4)=alpha-1;
UY(1,1)=alpha;
UY(1,2)=rho;
UE(1)=1;
%
% Consumption c(t)=E(c(t)|t-1)+eta(t)
%
UX(2,2)=1;
UY(2,3)=1;
UN(2)=1;
%
% Resource constraint
%
UX(3,1)=1;
UX(3,2)=-sc;
UX(3,3)=-si;
%
% Consumption-leisure arbitrage
%
UX(4,1)=-1;
UX(4,2)=1;
UX(4,4)=1;
%
% Accumulation of capital
%
G0Y(1,1)=1;
G1Y(1,1)=1-delta;
G1X(1,3)=delta;
%
% Productivity shock
%
G0Y(2,2)=1;
G1Y(2,2)=rho;
GE(2)=1;
%
% Euler equation
%
G0Y(3,1)=1-beta*(1-delta);
G0Y(3,3)=1;
G0X(3,1)=-(1-beta*(1-delta));
G1X(3,2)=1;
%
% Solution
%
% Step 1: solve the first set of equations
%
PIY = inv(UX)*UY;
PIE = inv(UX)*UE;
PIN = inv(UX)*UN;
%
% Step 2: build the standard System
%
A0  = G0Y+G0X*PIY;
A1  = G1Y+G1X*PIY;
B   = GE+G1X*PIE;
C   = GN+G1X*PIN;
%
% Step 3: Call Sims' routine
%
[MY,ME,ETA,MU_]=sims_solve(A0,A1,B,C);
%
% Step 4: Recover the impact function
%
PIE=PIE-PIN*ETA;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                            %
%               Impulse Response Functions                   %
%                                                            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
nrep    = 20;            % horizon of responses
YS      = zeros(3,nrep);
XS      = zeros(4,nrep);
Shock   = 1;
YS(:,1) = ME*Shock;
XS(:,1) = PIE;
for t=2:nrep;
    YS(:,t) = MY*YS(:,t-1);
    XS(:,t) = PIY*YS(:,t-1);
end
subplot(221);plot(XS(1,:));title('Output','fontname','times','fontsize',12);xlabel('Time','fontname','times','fontsize',12);set(gca,'fontname','times','fontsize',12);
subplot(222);plot(XS(2,:));title('Consumption','fontname','times','fontsize',12);xlabel('Time','fontname','times','fontsize',12);set(gca,'fontname','times','fontsize',12);
subplot(223);plot(XS(3,:));title('Investment','fontname','times','fontsize',12);xlabel('Time','fontname','times','fontsize',12);set(gca,'fontname','times','fontsize',12);
subplot(224);plot(XS(4,:));title('Hours worked','fontname','times','fontsize',12);xlabel('Time','fontname','times','fontsize',12);set(gca,'fontname','times','fontsize',12);

print -depsc2 irf_rbc.eps