function [PI,GAM,MSS,MSE,MSZ,MU_]=singsolve(Mcc,Mcs,Mss0,Mss1,Msc0,Msc1,Mse,bkth_,shth_,fwth_,stth_,tol_,print_)
%
% [PI,MSS,MSE,MU]=singsolve(Mcc,Mcs,Mss0,Mss1,Msc0,Msc1,Mse,bkth_,shth_,fwth_,stth_,tol_,print_)
%
% Solves system of the form :
%
% A0 Et X(t+1) = A1 X(t)  + A2 e(t+1)
%
% where A0 can be singular.
%
% Deals with indeterminacy
%
% bkth_  : vector containing the index of endogenous backward variables in the system
% shth_  : vector containing the index of exogenous backward variables in the system
% fwth_  : vector containing the index of forward variables in the system
% stth_  : vector containing the index of static variables in the system
%
% tol_   : tolerance parameter to determine static variables (1e-8 by default)
% print_ : when print_=1 (by default) prints some diagnostic information, else does not print anything
%
% Returns a solution of the form :
%
% Y(t)   = PI S(t)
% F(t)   = GAM S(t)
% S(t+1) = MSS S(t) + MSE e(t+1)
%
% MU contains the generalized eigenvalues of the system.
% 
% (c) F. Collard, August 1999  This routine is an adaptation of routines by P. Sderlind.
%

if (nargin<8)|isempty(tol_);
   tol_	= 1e-8;
end;

if (nargin<9)|isempty(print_);
   print_=1;
end;
PIT=inv(Mcc)*Mcs;
A0=Mss0-Msc0*PIT;
A1=-(Mss1-Msc1*PIT);
A2=Mse;
%clear Mcc Mcs Mss0 Mss1 Msc0 Msc1 Mse;

Order_		= [bkth_ shth_ fwth_ stth_];

A0				= A0(Order_,Order_);
A1				= A1(Order_,Order_);
A2				= A2(Order_,:);

%
% First we compute the Schur decomposition
%
[T,S,Q,Z]	= qz(A0,A1);
%
% We now reorder the eigenvalues in ascending order
%
nt = size(T,1);
i = 1;
while i<=nt-1;
   
   if abs(S(i,i)*T(i+1,i+1))>abs(T(i,i)*S(i+1,i+1));    
      
      a = T(i,i); d = S(i,i); b = T(i,i+1); e = S(i,i+1);
      c = T(i+1,i+1); f = S(i+1,i+1); 
      wz = [c*e-f*b, (c*d-f*a)'];
      xy = [(b*d-e*a)', (c*d-f*a)'];
      n = sqrt(wz*wz');
      m = sqrt(xy*xy');
      
      if n == 0
         return
      else
         wz = n\wz;
         xy = m\xy;
         wz = [wz; -wz(2)', wz(1)'];
         xy = [xy;-xy(2)', xy(1)'];
         T(i:i+1,:) = xy*T(i:i+1,:);
         S(i:i+1,:) = xy*S(i:i+1,:);
         T(:,i:i+1) = T(:,i:i+1)*wz;
         S(:,i:i+1) = S(:,i:i+1)*wz;
         Z(:,i:i+1) = Z(:,i:i+1)*wz;
         Q(i:i+1,:) = xy*Q(i:i+1,:);
      end
      
      if ~(i==1);
         i = i-2;
      end
      
   end
   
   i=i+1;
   
end
%
% We recover the diagonal of the S and T matrix in order to 
%
%	1) Determine the status of variables
%	2) compute the generalized eigenvalues
%
DS_			= diag(S);
DT_			= diag(T);
%
% Static variables
%
stat_			= find(abs(DT_)<=tol_);
%
% Dynamic variables 
%
dyn_			= find(abs(DT_)>tol_);		% Determine dynamic variables
MU_			= DS_(dyn_)./DT_(dyn_);		% Compute generalized eigenvalues
AMU_			= abs(MU_);						% Compute their modulus
forw_			= find(AMU_>1+tol_);			% Find the forward variables
forw_			= dyn_(forw_);					% Compute their index
back_			= find(AMU_<=1+tol_);			% Find the backward variables
back_			= dyn_(back_);					% Compute their index
nb_			= length(back_);
nf_			= length(forw_);
ny_			= length(stat_);

if ny_~=length([stth_]);
   PI=[];MSS=[];MSE=[];MSZ=[];
   error('Problem in the definition of your measured variables')
else   
   if (nf_==length(fwth_))&(nb_==(length(bkth_)+length(shth_)));
      if print_;
         disp('Conditions for a saddle path checked')
      end;
      B		= [1:nb_];
      F		= [nb_+1:nb_+nf_];
      Y		= [nb_+nf_+1:nb_+nf_+ny_];
      ZT	= Z';
      P		= ZT*A0;
      R		= ZT*A2;
      GAM	= -inv(ZT([F Y],[F Y]))*ZT([F Y],B);
      PI	= PIT(:,B)+PIT(:,[F Y])*GAM;
      M0I	= inv(A0(B,B)+A0(B,[F Y])*GAM(:,B));
      MSS	= M0I*(A1(B,B)+A1(B,[F Y])*GAM(:,B));
      PHI	= -inv(P([F Y],[F Y]))*R([F Y],:);
      MSE	= M0I*(A2(B,:)+A0(B,[F Y])*PHI);
      MSZ	= [];
   elseif (nf_<length(fwth_))&(nb_>(length(bkth_)+length(shth_)));
      ns_	= length(fwth_)-nf_;
      nf_	= nf_-ns_;
      if print_;
         if ns_==1;
            disp('Indeterminacy: 1 sunspot needed')
         else
            fprintf('Indeterminacy: %1.0f sunspots needed',nbsun_)
         end;
      end;
      nb_	= length(bkth_);
      B		= [1:nb_];
      S		= [nb_+1:nb_+ns_];
      F		= [nb_+ns_+1:nb_+ns_+nf_];
      Y		= [nb_+ns_+nf_+1:nb_+ns_+nf_+ny_];
      ZT	= Z';
      P		= ZT*A0;
      R		= ZT*A2;
      GAM	= -inv(ZT([F Y],[F Y]))*ZT([F Y],[B S]);
      PI	= PIT(:,[B S])+PIT(:,[F Y])*GAM;
      M0I	= inv(A0([B S],[B S])+A0([B S],[F Y])*GAM(:,[B S]));
      MSS	= M0I*(A1([B S],[B S])+A1([B S],[F Y])*GAM(:,[B S]));
      PHIE	= -inv(P([F Y],[F Y]))*R([F Y],:);
      PHIZ	= -inv(P([F Y],[F Y]))*P([F Y],S);
      MSE	= M0I*(A2([B S],:)+A0([B S],[F Y])*PHIE);
      MSZ	= M0I*(A0([B S],S)+A0([B S],[F Y])*PHIZ);
      
   else (nf_>length(fwth_))&(nb_<(length(bkth_)+length(shth_)));
      error('Source: No solution available')
   end
end
